﻿"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseConfig = void 0;
const tslib_1 = require("tslib");
const deepmerge_1 = tslib_1.__importDefault(require("deepmerge"));
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const errors_1 = require("../errors");
/** Parses and validates the config. */
function parseConfig(config = {}, root = process.cwd()) {
    const cwd = env('cwd', String) ?? config.cwd ?? root;
    const src = env('src', String) ?? config.src ?? 'src';
    if (typeof src !== 'string') {
        throw new errors_1.InvalidConfigError(`'src' must be a string: (${JSON.stringify(src)}). Read from ${envOrigin('src')}`, config);
    }
    let runtimePath = env('runtime_path', String) ?? config.runtimePath;
    if (!runtimePath) {
        try {
            runtimePath = path_1.default.join(
            // Joined @kitajs/runtime and generated separately because when
            // resolve is called on a package name (instead of folder if it was @kitajs/runtime/generated)
            // it will look only for the package.json and resolve from there.
            path_1.default.dirname(
            // Allows global installations to work
            require.resolve('@kitajs/runtime', { paths: [cwd] })), 'generated');
        }
        catch (error) {
            if (error.message.startsWith("Cannot find module '@kitajs/runtime'")) {
                throw new errors_1.RuntimeNotFoundError();
            }
            throw error;
        }
    }
    if (typeof runtimePath !== 'string') {
        throw new errors_1.InvalidConfigError(`'runtimePath' must be a string or undefined: (${JSON.stringify(runtimePath)}). Read from ${envOrigin('runtime_path')}`, config);
    }
    const declaration = env('declaration', Boolean) ?? config.declaration ?? true;
    if (typeof declaration !== 'boolean') {
        throw new errors_1.InvalidConfigError(`'declaration' must be a boolean: (${JSON.stringify(declaration)}). Read from ${envOrigin('declaration')}`, config);
    }
    const responses = env('responses', JSON.parse) ?? config.responses ?? {};
    if (typeof responses !== 'object') {
        throw new errors_1.InvalidConfigError(`'responses' must be an object: (${JSON.stringify(responses)}). Read from ${envOrigin('responses')}`, config);
    }
    const generatorConfig = (0, deepmerge_1.default)(env('generator_config', JSON.parse) ?? config.generatorConfig, {
        encodeRefs: false,
        sortProps: true,
        strictTuples: true,
        jsDoc: 'extended',
        parsers: [],
        formatters: []
    });
    if (typeof generatorConfig !== 'object') {
        throw new errors_1.InvalidConfigError(`'generatorConfig' must be an object: (${JSON.stringify(generatorConfig)}). Read from ${envOrigin('generator_config')}`, config);
    }
    const tsconfig = path_1.default.resolve(cwd, env('cwd', String) ?? config.tsconfig ?? 'tsconfig.json');
    if (!fs_1.default.existsSync(tsconfig)) {
        throw new errors_1.InvalidConfigError(`Tsconfig file ${JSON.stringify(tsconfig)} does not exist, are you on the correct directory?`, config);
    }
    const watchIgnore = env('watch_ignore', (x) => String(x).split(',')) ??
        config.watchIgnore ?? [path_1.default.join(cwd, 'node_modules'), path_1.default.join(cwd, 'dist'), runtimePath];
    if (!Array.isArray(watchIgnore)) {
        throw new errors_1.InvalidConfigError(`'watch.ignore' must be an array: (${JSON.stringify(watchIgnore)}). Read from ${envOrigin('watch_ignore')}`, config);
    }
    return {
        cwd,
        tsconfig,
        src: path_1.default.resolve(cwd, src),
        declaration: declaration,
        runtimePath: runtimePath,
        watchIgnore: watchIgnore,
        responses: responses,
        generatorConfig: generatorConfig,
        parameterParserAugmentor: config.parameterParserAugmentor || noop,
        providerParserAugmentor: config.providerParserAugmentor || noop,
        routeParserAugmentor: config.routeParserAugmentor || noop
    };
}
exports.parseConfig = parseConfig;
/** Reads and parses as json a environment variable. Following the format: `KITA_${name.toUpperCase()}` */
function env(name, cast) {
    const key = `KITA_${name.toUpperCase()}`;
    if (!process.env.hasOwnProperty(key)) {
        return undefined;
    }
    return cast(process.env[key]);
}
/** Returns the origin of a environment variable. Following the format: `KITA_${name.toUpperCase()}` */
function envOrigin(name) {
    return process.env[`KITA_${name.toUpperCase()}`] ? 'environment' : 'config';
}
/** No operation function */
function noop() { }
//# sourceMappingURL=parser.js.map