﻿"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UnknownNodeError = exports.UnknownKitaError = exports.KitaError = void 0;
const tslib_1 = require("tslib");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
/**
 * A KitaError instance is thrown when something goes wrong during the parsing, resolving or generation process.
 *
 * All errors that you can expect to be thrown by Kita are subclasses of this class.
 */
class KitaError extends Error {
    /** This property can be mutated at runtime by the user to indicate that the error has been resolved or handled. */
    suppress = false;
    /**
     * The error type.
     *
     * @example 'validator' | 'parser'
     */
    type;
    /**
     * The error code.
     *
     * This is a 3 digit number, where the first digit is the category of the error, and the last 2 digits are the error
     * code.
     *
     * | Code | Category                  |
     * | ---- | ------------------------- |
     * | 1    | General                   |
     * | 2    | Config                    |
     * | 3    | Parser                    |
     * | 4    | Validator                 |
     * | 5    | Formatter                 |
     * | 6    | ts-html-plugin (external) |
     */
    code;
    diagnostic;
    constructor(diagnostic) {
        super(typescript_1.default.flattenDiagnosticMessageText(diagnostic.messageText, '\n'));
        this.diagnostic = KitaError.createDiagnostic(diagnostic);
        this.code = this.diagnostic.code;
    }
    static createDiagnostic(diagnostic) {
        // Swap the node for the file, source, start and length properties
        if (diagnostic.node && diagnostic.node.pos !== -1) {
            diagnostic.file = diagnostic.node.getSourceFile();
            diagnostic.start = diagnostic.node.getStart();
            diagnostic.length = diagnostic.node.getWidth();
            diagnostic.node = undefined;
        }
        // @ts-expect-error - TODO: Should we keep as string or do some maths to get
        // the correct category?
        diagnostic.code = `kita - ${diagnostic.code}`;
        diagnostic.category ??= typescript_1.default.DiagnosticCategory.Error;
        return Object.assign({
            category: typescript_1.default.DiagnosticCategory.Error,
            file: undefined,
            length: 0,
            start: 0
        }, diagnostic, {
            __kita: true
        });
    }
}
exports.KitaError = KitaError;
/** A KitaError instance is thrown when something goes wrong during the parsing, resolving or generation process. */
class UnknownKitaError extends KitaError {
    data;
    constructor(message, data) {
        super({
            messageText: `Unknown error: ${message}`,
            code: -1
        });
        this.data = data;
    }
}
exports.UnknownKitaError = UnknownKitaError;
class UnknownNodeError extends KitaError {
    constructor(node, messageText) {
        super({
            code: -1,
            messageText,
            node
        });
    }
}
exports.UnknownNodeError = UnknownNodeError;
//# sourceMappingURL=base.js.map